//$Id: dialog-win.cc,v 1.11 2001/04/17 17:20:35 cactus Exp $ -*- c++ -*-

/* Guikachu Copyright (C) 2001 RDI Gerg <cactus@cactus.rulez.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "dialog-win.h"

#include "config.h"
#include <libgnome/libgnome.h>

#include <gtk--/menu.h>
#include <gnome--/dialog.h>
#include <sigc++/retbind.h>
#include "argumentsink.h"

using namespace Guikachu;

GUI::DialogWindow::DialogWindow (Resources::Dialog *res_):
    res (res_),
    help_combo (Resources::RESOURCE_STRING,
		SigC::slot (res, &Resources::Dialog::get_help_id),
		SigC::slot (res, &Resources::Dialog::set_help_id),
		res->changed),
    update_block (true)
{
    gui = glade_xml_new (GLADEDIR "/guikachu.glade", 0);

    using namespace Gtk;
    using namespace SigC;
    
    window = wrap (GTK_WINDOW (glade_xml_get_widget (gui, "res_dialog")));
    window->delete_event.connect (slot (this, &DialogWindow::delete_event_impl));


    // Widgets for page 1
    id_entry = wrap (GTK_ENTRY (glade_xml_get_widget (gui, "dialog_name")));
    title_entry = wrap (GTK_ENTRY (glade_xml_get_widget (gui, "dialog_title")));
    text_entry = wrap (GTK_TEXT (glade_xml_get_widget (gui, "dialog_text")));
    type_combo = wrap (GTK_OPTION_MENU (glade_xml_get_widget (gui, "dialog_type")));

    text_entry->set_word_wrap (true);

    // Set up callbacks
    id_entry->activate.connect (slot (this, &DialogWindow::id_entry_cb));
    id_entry->focus_out_event.connect (
	hide<int, GdkEventFocus*> (retbind (slot (this, &DialogWindow::id_entry_cb),0)));

    title_entry->activate.connect (slot (this, &DialogWindow::id_entry_cb));
    title_entry->focus_out_event.connect (
	hide<int, GdkEventFocus*> (retbind (slot (this, &DialogWindow::title_entry_cb),0)));

    text_entry->changed.connect (slot (this, &DialogWindow::text_entry_cb));

    // Dialog type dropdown
    Menu *type_menu = new Menu ();

#define DIALOG_TYPE(s,t) \
	    type_menu->items ().push_back ( \
		Menu_Helpers::MenuElem ( \
		    s, \
		    bind (slot (this, &DialogWindow::type_menu_cb), \
			  Resources::t)));

    DIALOG_TYPE (_("Information"),  DIALOG_INFORMATION);
    DIALOG_TYPE (_("Confirmation"), DIALOG_CONFIRMATION);
    DIALOG_TYPE (_("Warning"),      DIALOG_WARNING);
    DIALOG_TYPE (_("Error"),        DIALOG_ERROR);

#undef DIALOG_TYPE
    
    type_menu->show_all ();
    type_combo->set_menu (*manage (type_menu));

    Gtk::Container *help_place;
    help_place = wrap (GTK_CONTAINER (glade_xml_get_widget (gui, "dialog_help_place")));
    help_place->add (help_combo);
    
    // Widgets for page 2
    button_list = wrap (GTK_LIST (glade_xml_get_widget (gui, "dialog_buttonlist")));
    button_list->selection_changed.connect (slot (this, &DialogWindow::button_sel_change_cb));

    btnAdd    =  wrap (GTK_BUTTON (glade_xml_get_widget (gui, "dialog_btn_add")));
    btnUp     =  wrap (GTK_BUTTON (glade_xml_get_widget (gui, "dialog_btn_up")));
    btnDown   =  wrap (GTK_BUTTON (glade_xml_get_widget (gui, "dialog_btn_down")));
    btnRemove =  wrap (GTK_BUTTON (glade_xml_get_widget (gui, "dialog_btn_remove")));
    btnDefault = wrap (GTK_BUTTON (glade_xml_get_widget (gui, "dialog_btn_default")));

    btnAdd->clicked.connect     (slot (this, &DialogWindow::button_add_cb));
    btnUp->clicked.connect      (slot (this, &DialogWindow::button_up_cb));
    btnDown->clicked.connect    (slot (this, &DialogWindow::button_down_cb));
    btnRemove->clicked.connect  (slot (this, &DialogWindow::button_remove_cb));
    btnDefault->clicked.connect (slot (this, &DialogWindow::button_default_cb));
    
    res->changed.connect (slot (this, &DialogWindow::update));
    update ();
}

GUI::DialogWindow::~DialogWindow ()
{
    window->destroy ();
    gtk_object_unref (GTK_OBJECT (gui));
}

void GUI::DialogWindow::show ()
{
    window->show_all ();
    window->get_window ().raise ();
}

int GUI::DialogWindow::delete_event_impl (GdkEventAny *e)
{
    window->hide ();
    return true;
}

void GUI::DialogWindow::update ()
{
    gchar *title_buf = g_strdup_printf (_("Dialog: %s"), res->get_id ().c_str ());
    window->set_title (title_buf);
    g_free (title_buf);

    update_block = true;
    
    id_entry->set_text (res->get_id ());
    title_entry->set_text (res->get_title ());

    if (text_entry->get_length())
	text_entry->delete_text (0, text_entry->get_length ());
    text_entry->insert (res->get_text ());

    type_combo->set_history (res->get_dialog_type ());
    
    vector<string> buttons = res->get_buttons ();
    vector<string>::iterator i;
    unsigned int j;
    unsigned int default_button = res->get_default_button ();
    
    button_list->items().clear ();
    for (i = buttons.begin (), j = 0;
	 i != buttons.end (); i++, j++)
    {
	string label;

	if (j != default_button)
	    label = *i;
	else
	    label = "*" + *i + "*";
	
	Gtk::ListItem *item = new Gtk::ListItem (label);
	item->show ();
	item->set_user_data (GUINT_TO_POINTER (j));
	
	button_list->items().push_back (*manage (item));
    }
    
    update_block = false;
}

void GUI::DialogWindow::id_entry_cb ()
{
    res->set_id (id_entry->get_text ());
}

void GUI::DialogWindow::title_entry_cb ()
{
    res->set_title (title_entry->get_text ());
}

void GUI::DialogWindow::text_entry_cb ()
{
    if (update_block)
	return;

    int saved_pos = text_entry->get_position ();
    res->set_text (text_entry->get_chars (0, text_entry->get_length ()));
    gtk_editable_set_position (GTK_EDITABLE (text_entry->gtkobj ()), saved_pos);
}

void GUI::DialogWindow::type_menu_cb (Resources::DialogType type)
{
    res->set_dialog_type (type);
}

void GUI::DialogWindow::dialog_add_cb (string name)
{
    if (name != "")
    {
	vector<string> buttons = res->get_buttons ();	
	buttons.push_back (name);
	res->set_buttons (buttons);
    }
}

void GUI::DialogWindow::button_add_cb ()
{
    bool selection_saved = false;
    unsigned int index = 0;
    
    if (button_list->selection ().size () != 0)
    {
	Gtk::List::SelectionList::iterator sel = button_list->selection ().begin ();  
	index = GPOINTER_TO_UINT ((*sel)->get_user_data ());
	selection_saved = true;
    }
    
    Gnome::Dialog *dialog = Gnome::Dialogs::request
	(*window, false,
	 _("Enter name of new button"),
	 "", 0,
	 slot (this, &DialogWindow::dialog_add_cb));

    dialog->run_and_close ();

    if (selection_saved)
	button_list->select_item (index);
}

void GUI::DialogWindow::button_remove_cb ()
{
    g_return_if_fail (button_list->selection ().size () == 1);
    
    Gtk::List::SelectionList::iterator sel = button_list->selection ().begin ();  
    unsigned int index = GPOINTER_TO_UINT ((*sel)->get_user_data ());

    vector<string> buttons = res->get_buttons ();
    buttons.erase (buttons.begin () + index);
    res->set_buttons (buttons);

    if (index == res->get_default_button ())
	res->set_default_button (0);
}

void GUI::DialogWindow::button_default_cb ()
{
    g_return_if_fail (button_list->selection ().size () == 1);

    Gtk::List::SelectionList::iterator sel = button_list->selection ().begin ();
    unsigned int index = GPOINTER_TO_UINT ((*sel)->get_user_data ());

    res->set_default_button (index);

    button_list->select_item (index);
}

void GUI::DialogWindow::button_up_cb ()
{
    Gtk::List::SelectionList sel_list = button_list->selection ();
    g_return_if_fail (sel_list.size () == 1);

    unsigned int index = GPOINTER_TO_UINT ((*(sel_list.begin ()))->get_user_data ());
    g_return_if_fail (index != 0);

    vector<string> buttons = res->get_buttons ();
    vector<string>::iterator curr = buttons.begin () + index;
    vector<string>::iterator prev = curr - 1;
    iter_swap (curr, prev);    
    res->set_buttons (buttons);

    unsigned int old_default = res->get_default_button ();

    if (index == old_default)
	res->set_default_button (index - 1);
    if (index - 1 == old_default)
	res->set_default_button (index);

    button_list->select_item (index - 1);
}

void GUI::DialogWindow::button_down_cb ()
{
    Gtk::List::SelectionList sel_list = button_list->selection ();
    g_return_if_fail (sel_list.size () != 0);

    unsigned int index = GPOINTER_TO_UINT ((*(sel_list.begin ()))->get_user_data ());
    g_return_if_fail (index < button_list->items ().size () - 1);

    vector<string> buttons = res->get_buttons ();
    vector<string>::iterator curr = buttons.begin () + index;
    vector<string>::iterator next = curr + 1;
    iter_swap (curr, next);
    res->set_buttons (buttons);

    unsigned int old_default = res->get_default_button ();

    if (index == old_default)
	res->set_default_button (index + 1);
    if (index + 1 == old_default)
	res->set_default_button (index);

    button_list->select_item (index + 1);
}

void GUI::DialogWindow::button_sel_change_cb ()
{
    Gtk::List::SelectionList sel_list = button_list->selection ();

    if (!sel_list.size ())
    {
	
	btnRemove->set_sensitive (false);
	btnDefault->set_sensitive (false);
	btnUp->set_sensitive (false);
	btnDown->set_sensitive (false);
	
    } else {
	
	g_return_if_fail (sel_list.size() == 1);

	unsigned int index = GPOINTER_TO_UINT ((*(sel_list.begin()))->get_user_data ());
    
	btnRemove->set_sensitive (true);
	btnDefault->set_sensitive (true);
	btnUp->set_sensitive ((index > 0));
	btnDown->set_sensitive ((index < (button_list->items ().size () - 1)));
    }
}
